@@ -105,7 +105,7 @@ get_collapse <- function(opts = NULL) if(is.null(opts)) as.list(.op) else if(len
                             "%r-%", "%r*%", "%r/%", "%r+%", "%rr%", "add_stub", "add_vars",
                             "add_vars<-", "all_funs", "all_identical", "all_obj_equal", "allNA",
                             "alloc", "allv", "any_duplicated", "anyv", "as_character_factor",
-                            "as_factor_GRP", "as_factor_qG", "as_numeric_factor", "as.character_factor",
+                            "as_factor_GRP", "as_factor_qG", "as_numeric_factor", "as_integer_factor", "as.character_factor",
                             "as.factor_GRP", "as.factor_qG", "as.numeric_factor", "atomic_elem",
                             "atomic_elem<-", "av", "av<-", "B", "BY", "BY.data.frame", "BY.default",
                             "BY.matrix", "cat_vars", "cat_vars<-", "char_vars", "char_vars<-",
@@ -119,7 +119,7 @@ get_collapse <- function(opts = NULL) if(is.null(opts)) as.list(.op) else if(len
                             "fdiff.default", "fdiff.matrix", "fdim", "fdist", "fdroplevels",
                             "fdroplevels.data.frame", "fdroplevels.factor", "fduplicated",
                             "ffirst", "ffirst.data.frame", "ffirst.default", "ffirst.matrix",
-                            "fFtest", "fFtest.default", "fgroup_by", "fgroup_vars", "fgrowth",
+                            "fFtest", "fFtest.default", "fgroup_by", "group_by_vars", "fgroup_vars", "fgrowth",
                             "fgrowth.data.frame", "fgrowth.default", "fgrowth.matrix", "fhdbetween",
                             "fHDbetween", "fhdbetween.data.frame", "fhdbetween.default",
                             "fhdbetween.matrix", "fhdwithin", "fHDwithin", "fhdwithin.data.frame",
@@ -152,7 +152,7 @@ get_collapse <- function(opts = NULL) if(is.null(opts)) as.list(.op) else if(len
                             "is.categorical", "is.Date", "is.GRP", "is.qG", "is.unlistable",
                             "itn", "ix", "join", "L", "ldepth", "list_elem", "list_elem<-",
                             "logi_vars", "logi_vars<-", "massign", "mctl", "missing_cases",
-                            "mrtl", "mtt", "na_insert", "na_omit", "na_rm", "namlab", "num_vars",
+                            "mrtl", "mtt", "na_insert", "na_omit", "na_rm", "na_locf", "na_focb", "namlab", "num_vars",
                             "num_vars<-", "nv", "nv<-", "pad", "pivot", "plot.psmat", "print.pwcor",
                             "print.pwcov", "print.qsu", "psacf", "psacf.data.frame", "psacf.default",
                             "psccf", "psccf.default", "psmat", "psmat.data.frame", "psmat.default",
@@ -177,11 +177,11 @@ get_collapse <- function(opts = NULL) if(is.null(opts)) as.list(.op) else if(len
 .COLLAPSE_ALL <- sort(unique(c("%-=%", "%!=%", "%!iin%", "%!in%", "%*=%", "%/=%", "%+=%", "%=%", "%==%", "%c-%", "%c*%", "%c/%", "%c+%",
                                "%cr%", "%iin%", "%r-%", "%r*%", "%r/%", "%r+%", "%rr%", "add_stub", "add_vars", "add_vars<-", "all_funs",
                                "all_identical", "all_obj_equal", "allNA", "alloc", "allv", "any_duplicated", "anyv", "as_character_factor",
-                               "as_factor_GRP", "as_factor_qG", "as_numeric_factor", "atomic_elem", "atomic_elem<-", "av", "av<-", "B", "BY",
+                               "as_factor_GRP", "as_factor_qG", "as_numeric_factor", "as_integer_factor", "atomic_elem", "atomic_elem<-", "av", "av<-", "B", "BY",
                                "cat_vars", "cat_vars<-", "char_vars", "char_vars<-", "cinv", "ckmatch", "collap", "collapg", "collapv", "colorder",
                                "colorderv", "copyAttrib", "copyMostAttrib", "copyv", "D", "dapply", "date_vars", "Date_vars", "date_vars<-",
                                "Date_vars<-", "descr", "Dlog", "fact_vars", "fact_vars<-", "fbetween", "fcompute", "fcomputev", "fcount",
-                               "fcountv", "fcumsum", "fdiff", "fdim", "fdist", "fdroplevels", "fduplicated", "ffirst", "fFtest", "fgroup_by",
+                               "fcountv", "fcumsum", "fdiff", "fdim", "fdist", "fdroplevels", "fduplicated", "ffirst", "fFtest", "fgroup_by", "group_by_vars",
                                "fgroup_vars", "fgrowth", "fhdbetween", "fhdwithin", "findex", "findex_by", "finteraction", "flag", "flast", "flm",
                                "fmatch", "fmax", "fmean", "fmedian", "fmin", "fmode", "fmutate", "fncol", "fndistinct", "fnlevels", "fnobs", "fnrow",
                                "fnth", "fnunique", "fprod", "fquantile", "frange", "frename", "fscale", "fsd", "fselect", "fselect<-", "fsubset", "fsum",
@@ -189,7 +189,7 @@ get_collapse <- function(opts = NULL) if(is.null(opts)) as.list(.op) else if(len
                                "gby", "get_collapse", "get_elem", "get_vars", "get_vars<-", "GGDC10S", "greorder", "group", "groupid", "GRP", "GRPid",
                                "GRPN", "GRPnames", "gsplit", "gv", "gv<-", "gvr", "gvr<-", "has_elem", "HDB", "HDW", "iby", "irreg_elem", "is_categorical",
                                "is_date", "is_GRP", "is_irregular", "is_qG", "is_unlistable", "itn", "ix", "join", "L", "ldepth", "list_elem", "list_elem<-",
-                               "logi_vars", "logi_vars<-", "massign", "mctl", "missing_cases", "mrtl", "mtt", "na_insert", "na_omit", "na_rm", "namlab",
+                               "logi_vars", "logi_vars<-", "massign", "mctl", "missing_cases", "mrtl", "mtt", "na_insert", "na_omit", "na_rm", "na_locf", "na_focb", "namlab",
                                "num_vars", "num_vars<-", "nv", "nv<-", "pad", "pivot", "psacf", "psccf", "psmat", "pspacf", "pwcor", "pwcov", "pwnobs",
                                "qDF", "qDT", "qF", "qG", "qM", "qsu", "qtab", "qtable", "qTBL", "radixorder", "radixorderv", "rapply2d", "recode_char",
                                "recode_num", "reg_elem", "reindex", "relabel", "replace_inf", "replace_Inf", "replace_na", "replace_NA", "replace_outliers",
