@@ -59,7 +59,7 @@ join <- function(x, y,
 
   # Matching step
   rjoin <- switch(how, right = TRUE, FALSE)
-  count <- verbose || validate != "m:m"
+  count <- verbose || validate != "m:m" || length(attr)
 
   if(sort) {
     if(rjoin) {
@@ -105,6 +105,7 @@ join <- function(x, y,
     g <- group(if(rjoin) x[ixon] else y[iyon], group.sizes = TRUE)
     m <- multi_match(m, g)
     if(is.list(m)) {
+      multiple <- 2L
       if(rjoin) y <- .Call(C_subsetDT, y, m[[1L]], seq_along(y), FALSE)
       else x <- .Call(C_subsetDT, x, m[[1L]], seq_along(x), FALSE)
       m <- m[[2L]]
@@ -179,7 +180,7 @@ join <- function(x, y,
   # Core: do the joins
   res <- switch(how,
     left = {
-      y_res <- .Call(C_subsetDT, y, m, seq_along(y)[-iyon], if(count) attr(m, "N.nomatch") else TRUE)
+      y_res <- if(identical(unattrib(m), seq_row(y))) y[-iyon] else .Call(C_subsetDT, y, m, seq_along(y)[-iyon], if(count) attr(m, "N.nomatch") else TRUE)
       c(x, y_res)
     },
     inner = {
@@ -192,7 +193,7 @@ join <- function(x, y,
         # if(length(rn)) ax[["row.names"]] <- if(is.numeric(rn) || is.null(rn) || rn[1L] == "1")
         #             .set_row_names(length(x_ind)) else Csv(rn, x_ind)
       }
-      y_res <- .Call(C_subsetDT, y, m, seq_along(y)[-iyon], FALSE)
+      y_res <- if(identical(unattrib(m), seq_row(y))) y[-iyon] else .Call(C_subsetDT, y, m, seq_along(y)[-iyon], FALSE)
       c(x, y_res)
     },
     full = {
@@ -208,7 +209,7 @@ join <- function(x, y,
         }
       }
       if(cond) { # TODO: special case ? 1 distinct value etc.??
-        tind <- seq_len(tsize)[-um] # TODO: Table may not be unique.
+        tind <- if(length(um)) seq_len(tsize)[-um] else seq_len(tsize) # TODO: Table may not be unique.
         res_nrow <- length(m) + length(tind)
         x_res <- .Call(C_subsetDT, x, seq_len(res_nrow), seq_along(x)[-ixon], TRUE)  # Need check here because oversize indices !!
         y_res <- .Call(C_subsetDT, y, vec(list(m, tind)), seq_along(y)[-iyon], TRUE) # Need check here because oversize indices !!
@@ -223,13 +224,13 @@ join <- function(x, y,
           c(on_res, x_res, y_res)
         }
       } else { # If all elements of table are matched, this is simply a left join
-        how <- "left"
-        y_res <- .Call(C_subsetDT, y, m, seq_along(y)[-iyon], if(count) attr(m, "N.nomatch") else TRUE) # anyNA(um) ??
+        how <- if(multiple == 2L) "left_setrn" else "left"
+        y_res <- if(identical(unattrib(m), seq_row(y))) y[-iyon] else .Call(C_subsetDT, y, m, seq_along(y)[-iyon], if(count) attr(m, "N.nomatch") else TRUE) # anyNA(um) ??
         c(x, y_res)
       }
     },
     right = {
-      x_res <- .Call(C_subsetDT, x, m, seq_along(x)[-ixon], if(count) attr(m, "N.nomatch") else TRUE)
+      x_res <- if(identical(unattrib(m), seq_row(x))) x[-ixon] else .Call(C_subsetDT, x, m, seq_along(x)[-ixon], if(count) attr(m, "N.nomatch") else TRUE)
       # if(length(ax[["row.names"]])) ax[["row.names"]] <- .set_row_names(length(m))
       y_on <- y[iyon]
       names(y_on) <- xon
